__version__ = '0.4.2.1'

from .api_fcatory import APIFactory
from .model import *
from .ebest import *
from .kis import *

__all__ = ['APIFactory', 'model',]


APIS = ["EBEST", "KIS", "LS"]

def print_version_info():
    print(f"The version of this stock finance API is {__version__}.")

def create_api(api_type):
    return APIFactory.create_api(api_type)


from dataclasses import dataclass, field
from typing import List

@dataclass(frozen=True)
class Price:
    workday: str
    code: str
    price: int
    open: int
    high: int
    low: int
    close: int
    volume: int
    volume_amt: int

@dataclass(frozen=True)
class Hoga:
    price: int
    qty: int

@dataclass(frozen=True)
class OrderBook:
    code: str
    total_buy: int
    total_sell: int
    buy: List[Hoga] = field(default_factory=list)
    sell: List[Hoga] = field(default_factory=list)


from abc import ABC, abstractmethod


class IAPI(ABC):
    @abstractmethod
    def oauth(self):
        raise NotImplemented

    @abstractmethod
    def set_oauth_info(self, app_key, app_secret):
        raise NotImplemented

    @abstractmethod
    def set_access_token(self, token):
        raise NotImplemented

    @abstractmethod
    def get_ohlcv(self, code, frdate, todate):
        raise NotImplemented

    @abstractmethod
    def get_index(self, code, frdate, todate):
        raise NotImplemented

    @abstractmethod
    def get_orderbook(self, code):
        raise NotImplemented

    @abstractmethod
    def set_queue(self, queue, condition):
        raise NotImplemented

    @abstractmethod
    def set_price_queue(self, price_queue, condition):
        raise NotImplemented

    @abstractmethod
    def set_trade_queue(self, trade_queue, condition):
        raise NotImplemented

    @abstractmethod
    def set_orderbook_queue(self, orderbook_queue, condition):
        raise NotImplemented

    @abstractmethod
    async def recv_price(self, code):
        raise NotImplemented

    @abstractmethod
    async def recv_index(self, code):
        raise NotImplemented

    @abstractmethod
    async def recv_orderbook(self, code):
        raise NotImplemented

    @abstractmethod
    async def recv_trade(self, code):
        raise NotImplemented


from enum import Enum

class TRADE_FLAG(Enum):
    ORDER = 1
    MODIFY = 2
    CANCLE = 3
    COMPLETE = 4

class ORDER_FLAG(Enum):
    BUY = 1
    SELL = 2
    VIEW = 3

class MARKET_FLAG(Enum):
    KOSPI = 1
    KOSDAQ = 2
    #ETF = 3
    #ETN = 4